@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
+ * Copyright (c) 2000, 2014 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,10 +12,20 @@
  *								bug 367203 - [compiler][null] detect assigning null to nonnull argument
  *								bug 365519 - editorial cleanup after bug 186342 and bug 365387
  *								bug 365531 - [compiler][null] investigate alternative strategy for internally encoding nullness defaults
+ *								bug 382353 - [1.8][compiler] Implementation property modifiers should be accepted on default methods.
+ *								bug 392099 - [1.8][compiler][null] Apply null annotation on types for null analysis
  *								bug 388281 - [compiler][null] inheritance of null annotations as an option
+ *								bug 401030 - [1.8][null] Null analysis support for lambda methods.
+ *								Bug 392099 - [1.8][compiler][null] Apply null annotation on types for null analysis
+ *								Bug 392238 - [1.8][compiler][null] Detect semantically invalid null type annotations
+ *								Bug 403216 - [1.8][null] TypeReference#captureTypeAnnotations treats type annotations as type argument annotations
+ *								Bug 417295 - [1.8[[null] Massage type annotated null analysis to gel well with deep encoded type bindings.
+ *								Bug 392238 - [1.8][compiler][null] Detect semantically invalid null type annotations
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
+import java.util.List;
+
 import org.eclipse.jdt.core.compiler.*;
 import org.eclipse.jdt.internal.compiler.*;
 import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
@@ -27,6 +37,7 @@ import org.eclipse.jdt.internal.compiler.problem.*;
 import org.eclipse.jdt.internal.compiler.parser.*;
 import org.eclipse.jdt.internal.compiler.util.Util;
 
+@SuppressWarnings({"rawtypes"})
 public abstract class AbstractMethodDeclaration
 	extends ASTNode
 	implements ProblemSeverities, ReferenceContext {
@@ -40,6 +51,8 @@ public abstract class AbstractMethodDeclaration
 	public int modifiers;
 	public int modifiersSourceStart;
 	public Annotation[] annotations;
+	// jsr 308
+	public Receiver receiver;
 	public Argument[] arguments;
 	public TypeReference[] thrownExceptions;
 	public Statement[] statements;
@@ -79,17 +92,25 @@ public abstract class AbstractMethodDeclaration
 	 * we create the argument binding and resolve annotations in order to compute null annotation tagbits.
 	 */
 	public void createArgumentBindings() {
-		if (this.arguments != null && this.binding != null) {
-			for (int i = 0, length = this.arguments.length; i < length; i++) {
-				Argument argument = this.arguments[i];
-				argument.createBinding(this.scope, this.binding.parameters[i]);
+		createArgumentBindings(this.arguments, this.binding, this.scope);
+	}
+	// version for invocation from LambdaExpression:
+	static void createArgumentBindings(Argument[] arguments, MethodBinding binding, MethodScope scope) {
+		boolean useTypeAnnotations = scope.compilerOptions().sourceLevel >= ClassFileConstants.JDK1_8;
+		if (arguments != null && binding != null) {
+			for (int i = 0, length = arguments.length; i < length; i++) {
+				Argument argument = arguments[i];
+				binding.parameters[i] = argument.createBinding(scope, binding.parameters[i]);
+				if (useTypeAnnotations)
+					continue; // no business with SE7 null annotations in the 1.8 case.
 				// createBinding() has resolved annotations, now transfer nullness info from the argument to the method:
-				if ((argument.binding.tagBits & (TagBits.AnnotationNonNull|TagBits.AnnotationNullable)) != 0) {
-					if (this.binding.parameterNonNullness == null) {
-						this.binding.parameterNonNullness = new Boolean[this.arguments.length];
-						this.binding.tagBits |= TagBits.IsNullnessKnown;
+				long argTypeTagBits = (argument.binding.tagBits & TagBits.AnnotationNullMASK);
+				if (argTypeTagBits != 0) {
+					if (binding.parameterNonNullness == null) {
+						binding.parameterNonNullness = new Boolean[arguments.length];
+						binding.tagBits |= TagBits.IsNullnessKnown;
 					}
-					this.binding.parameterNonNullness[i] = Boolean.valueOf((argument.binding.tagBits & TagBits.AnnotationNonNull) != 0);
+					binding.parameterNonNullness[i] = Boolean.valueOf(argTypeTagBits == TagBits.AnnotationNonNull);
 				}
 			}
 		}
@@ -112,7 +133,7 @@ public abstract class AbstractMethodDeclaration
 			AnnotationBinding[][] paramAnnotations = null;
 			for (int i = 0, length = this.arguments.length; i < length; i++) {
 				Argument argument = this.arguments[i];
-				argument.bind(this.scope, this.binding.parameters[i], used);
+				this.binding.parameters[i] = argument.bind(this.scope, this.binding.parameters[i], used);
 				if (argument.annotations != null) {
 					if (paramAnnotations == null) {
 						paramAnnotations = new AnnotationBinding[length][];
@@ -174,21 +195,42 @@ public abstract class AbstractMethodDeclaration
 	/**
 	 * Feed null information from argument annotations into the analysis and mark arguments as assigned.
 	 */
-	void analyseArguments(FlowInfo flowInfo) {
-		if (this.arguments != null) {
-			for (int i = 0, count = this.arguments.length; i < count; i++) {
-				if (this.binding.parameterNonNullness != null) {
+	static void analyseArguments(FlowInfo flowInfo, Argument[] methodArguments, MethodBinding methodBinding) {
+		if (methodArguments != null) {
+			for (int i = 0, count = methodArguments.length; i < count; i++) {
+				if (methodBinding.parameterNonNullness != null) {
 					// leverage null-info from parameter annotations:
-					Boolean nonNullNess = this.binding.parameterNonNullness[i];
+					Boolean nonNullNess = methodBinding.parameterNonNullness[i];
 					if (nonNullNess != null) {
 						if (nonNullNess.booleanValue())
-							flowInfo.markAsDefinitelyNonNull(this.arguments[i].binding);
+							flowInfo.markAsDefinitelyNonNull(methodArguments[i].binding);
 						else
-							flowInfo.markPotentiallyNullBit(this.arguments[i].binding);
+							flowInfo.markPotentiallyNullBit(methodArguments[i].binding);
 					}
 				}
 				// tag parameters as being set:
-				flowInfo.markAsDefinitelyAssigned(this.arguments[i].binding);
+				flowInfo.markAsDefinitelyAssigned(methodArguments[i].binding);
+			}
+		}
+	}
+
+	/**
+	 * Feed null information from argument annotations into the analysis and mark arguments as assigned.
+	 * Variant for Java 8 using type annotations
+	 */
+	static void analyseArguments18(FlowInfo flowInfo, Argument[] methodArguments, MethodBinding methodBinding) {
+		if (methodArguments != null) {
+			int length = Math.min(methodBinding.parameters.length, methodArguments.length);
+			for (int i = 0; i < length; i++) {
+				// leverage null type annotations:
+				long tagBits = methodBinding.parameters[i].tagBits & TagBits.AnnotationNullMASK;
+				if (tagBits == TagBits.AnnotationNonNull)
+					flowInfo.markAsDefinitelyNonNull(methodArguments[i].binding);
+				else if (tagBits == TagBits.AnnotationNullable)
+					flowInfo.markPotentiallyNullBit(methodArguments[i].binding);
+	
+				// tag parameters as being set:
+				flowInfo.markAsDefinitelyAssigned(methodArguments[i].binding);
 			}
 		}
 	}
@@ -304,6 +346,10 @@ public abstract class AbstractMethodDeclaration
 		classFile.completeMethodInfo(this.binding, methodAttributeOffset, attributeNumber);
 	}
 
+	public void getAllAnnotationContexts(int targetType, List allAnnotationContexts) {
+		// do nothing
+	}
+
 	private void checkArgumentsSize() {
 		TypeBinding[] parameters = this.binding.parameters;
 		int size = 1; // an abstract method or a native method cannot be static
@@ -361,6 +407,10 @@ public abstract class AbstractMethodDeclaration
 		return false;
 	}
 
+	public boolean isDefaultMethod() {
+		return false;
+	}
+
 	public boolean isInitializationMethod() {
 
 		return false;
@@ -399,7 +449,10 @@ public abstract class AbstractMethodDeclaration
 		}
 		printIndent(tab, output);
 		printModifiers(this.modifiers, output);
-		if (this.annotations != null) printAnnotations(this.annotations, output);
+		if (this.annotations != null) {
+			printAnnotations(this.annotations, output);
+			output.append(' ');
+		}
 
 		TypeParameter[] typeParams = typeParameters();
 		if (typeParams != null) {
@@ -414,9 +467,12 @@ public abstract class AbstractMethodDeclaration
 		}
 
 		printReturnType(0, output).append(this.selector).append('(');
+		if (this.receiver != null) {
+			this.receiver.print(0, output);
+		}
 		if (this.arguments != null) {
 			for (int i = 0; i < this.arguments.length; i++) {
-				if (i > 0) output.append(", "); //$NON-NLS-1$
+				if (i > 0 || this.receiver != null) output.append(", "); //$NON-NLS-1$
 				this.arguments[i].print(0, output);
 			}
 		}
@@ -462,16 +518,20 @@ public abstract class AbstractMethodDeclaration
 
 		try {
 			bindArguments();
+			resolveReceiver();
 			bindThrownExceptions();
 			resolveJavadoc();
 			resolveAnnotations(this.scope, this.annotations, this.binding);
-			validateNullAnnotations();
+			
+			long sourceLevel = this.scope.compilerOptions().sourceLevel;
+			validateNullAnnotations(sourceLevel);
+
 			resolveStatements();
 			// check @Deprecated annotation presence
 			if (this.binding != null
 					&& (this.binding.getAnnotationTagBits() & TagBits.AnnotationDeprecated) == 0
 					&& (this.binding.modifiers & ClassFileConstants.AccDeprecated) != 0
-					&& this.scope.compilerOptions().sourceLevel >= ClassFileConstants.JDK1_5) {
+					&& sourceLevel >= ClassFileConstants.JDK1_5) {
 				this.scope.problemReporter().missingDeprecatedAnnotationForMethod(this);
 			}
 		} catch (AbortMethod e) {
@@ -480,6 +540,55 @@ public abstract class AbstractMethodDeclaration
 		}
 	}
 
+	public void resolveReceiver() {
+		if (this.receiver == null) return;
+
+		if (this.receiver.modifiers != 0) {
+			this.scope.problemReporter().illegalModifiers(this.receiver.declarationSourceStart, this.receiver.declarationSourceEnd);
+		}
+
+		TypeBinding resolvedReceiverType = this.receiver.type.resolvedType;
+		if (this.binding == null || resolvedReceiverType == null || !resolvedReceiverType.isValidBinding()) {
+			return;
+		}
+
+		ReferenceBinding declaringClass = this.binding.declaringClass;
+		/* neither static methods nor methods in anonymous types can have explicit 'this' */
+		if (this.isStatic() || declaringClass.isAnonymousType()) {
+			this.scope.problemReporter().disallowedThisParameter(this.receiver);
+			return; // No need to do further validation
+		}
+
+		ReferenceBinding enclosingReceiver = this.scope.enclosingReceiverType();
+		if (this.isConstructor()) {
+			/* Only non static member types or local types can declare explicit 'this' params in constructors */
+			if (declaringClass.isStatic()
+					|| (declaringClass.tagBits & (TagBits.IsLocalType | TagBits.IsMemberType)) == 0) { /* neither member nor local type */
+				this.scope.problemReporter().disallowedThisParameter(this.receiver);
+				return; // No need to do further validation
+			}
+			enclosingReceiver = enclosingReceiver.enclosingType();
+		}
+
+		char[][] tokens = (this.receiver.qualifyingName == null) ? null : this.receiver.qualifyingName.getName();
+		if (this.isConstructor()) {
+			if (tokens == null || tokens.length > 1 || !CharOperation.equals(enclosingReceiver.sourceName(), tokens[0])) {
+				this.scope.problemReporter().illegalQualifierForExplicitThis(this.receiver, enclosingReceiver);
+				this.receiver.qualifyingName = null;
+			}
+		} else if (tokens != null && tokens.length > 0) {
+			this.scope.problemReporter().illegalQualifierForExplicitThis2(this.receiver);
+			this.receiver.qualifyingName = null;
+		}
+
+		if (TypeBinding.notEquals(enclosingReceiver, resolvedReceiverType)) {
+			this.scope.problemReporter().illegalTypeForExplicitThis(this.receiver, enclosingReceiver);
+		}
+
+		if (resolvedReceiverType.hasNullTypeAnnotations()) {
+			this.scope.problemReporter().nullAnnotationUnsupportedLocation(this.receiver.type);
+		}
+	}
 	public void resolveJavadoc() {
 
 		if (this.binding == null) return;
@@ -519,6 +628,10 @@ public abstract class AbstractMethodDeclaration
 	public void tagAsHavingErrors() {
 		this.ignoreFurtherInvestigation = true;
 	}
+	
+	public void tagAsHavingIgnoredMandatoryErrors(int problemId) {
+		// Nothing to do for this context;
+	}
 
 	public void traverse(
 		ASTVisitor visitor,
@@ -530,17 +643,28 @@ public abstract class AbstractMethodDeclaration
 	    return null;
 	}
 
-	void validateNullAnnotations() {
+	void validateNullAnnotations(long sourceLevel) {
+		if (this.binding == null) return;
 		// null annotations on parameters?
-		if (this.binding != null && this.binding.parameterNonNullness != null) {
-			int length = this.binding.parameters.length;
-			for (int i=0; i<length; i++) {
-				if (this.binding.parameterNonNullness[i] != null) {
-					long nullAnnotationTagBit =  this.binding.parameterNonNullness[i].booleanValue()
-							? TagBits.AnnotationNonNull : TagBits.AnnotationNullable;
-					this.scope.validateNullAnnotation(nullAnnotationTagBit, this.arguments[i].type, this.arguments[i].annotations);
+		if (sourceLevel < ClassFileConstants.JDK1_8) {
+			if (this.binding.parameterNonNullness != null) {
+				int length = this.binding.parameters.length;
+				for (int i=0; i<length; i++) {
+					if (this.binding.parameterNonNullness[i] != null) {
+						long nullAnnotationTagBit =  this.binding.parameterNonNullness[i].booleanValue()
+								? TagBits.AnnotationNonNull : TagBits.AnnotationNullable;
+						if (!this.scope.validateNullAnnotation(nullAnnotationTagBit, this.arguments[i].type, this.arguments[i].annotations))
+							this.binding.parameterNonNullness[i] = null;
+					}
 				}
 			}
+		} else {
+			int length = this.binding.parameters.length;
+			for (int i=0; i<length; i++) {
+				this.scope.validateNullAnnotation(this.binding.parameters[i].tagBits, this.arguments[i].type, this.arguments[i].annotations);
+// TODO(stephan) remove once we're sure:
+//					this.binding.parameters[i] = this.binding.parameters[i].unannotated();
+			}			
 		}
 	}
 }
