@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
+ * Copyright (c) 2000, 2014 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -8,6 +8,8 @@
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Stephan Herrmann  - Contribution for bug 295551
+ *     Jesper S Moller   - Contributions for
+ *							  Bug 405066 - [1.8][compiler][codegen] Implement code generation infrastructure for JSR335             
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -38,6 +40,7 @@ import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
 import org.eclipse.jdt.internal.compiler.problem.ProblemSeverities;
 import org.eclipse.jdt.internal.compiler.util.HashSetOfInt;
 
+@SuppressWarnings({ "rawtypes", "unchecked" })
 public class CompilationUnitDeclaration extends ASTNode implements ProblemSeverities, ReferenceContext {
 
 	private static final Comparator STRING_LITERAL_COMPARATOR = new Comparator() {
@@ -72,10 +75,14 @@ public class CompilationUnitDeclaration extends ASTNode implements ProblemSeveri
 	private int stringLiteralsPtr;
 	private HashSetOfInt stringLiteralsStart;
 
+	public boolean[] validIdentityComparisonLines;
+
 	IrritantSet[] suppressWarningIrritants;  // irritant for suppressed warnings
 	Annotation[] suppressWarningAnnotations;
 	long[] suppressWarningScopePositions; // (start << 32) + end
 	int suppressWarningsCount;
+	public int functionalExpressionsCount;
+	public FunctionalExpression[] functionalExpressions;
 
 public CompilationUnitDeclaration(ProblemReporter problemReporter, CompilationResult compilationResult, int sourceLength) {
 	this.problemReporter = problemReporter;
@@ -145,6 +152,7 @@ public void cleanUp() {
 		// null out the classfile backpointer to a type binding
 		classFile.referenceBinding = null;
 		classFile.innerClassesBindings = null;
+		classFile.bootstrapMethods = null;
 		classFile.missingTypes = null;
 		classFile.visitedTypes = null;
 	}
@@ -416,6 +424,10 @@ public boolean isSuppressed(CategorizedProblem problem) {
 	return false;
 }
 
+public boolean hasFunctionalTypes() {
+	return this.compilationResult.hasFunctionalTypes;
+}
+
 public boolean hasErrors() {
 	return this.ignoreFurtherInvestigation;
 }
@@ -525,6 +537,20 @@ public void record(LocalTypeBinding localType) {
 	this.localTypes[this.localTypeCount++] = localType;
 }
 
+/*
+ * Keep track of all lambda/method reference expressions, so as to be able to look it up later without 
+ * having to traverse AST. Return the 1 based "ordinal" in the CUD.
+ */
+public int record(FunctionalExpression expression) {
+	if (this.functionalExpressionsCount == 0) {
+		this.functionalExpressions = new FunctionalExpression[5];
+	} else if (this.functionalExpressionsCount == this.functionalExpressions.length) {
+		System.arraycopy(this.functionalExpressions, 0, (this.functionalExpressions = new FunctionalExpression[this.functionalExpressionsCount * 2]), 0, this.functionalExpressionsCount);
+	}
+	this.functionalExpressions[this.functionalExpressionsCount] = expression;
+	return ++this.functionalExpressionsCount;
+}
+
 public void resolve() {
 	int startingTypeIndex = 0;
 	boolean isPackageInfo = isPackageInfo();
@@ -672,8 +698,15 @@ public void tagAsHavingErrors() {
 	this.ignoreFurtherInvestigation = true;
 }
 
+public void tagAsHavingIgnoredMandatoryErrors(int problemId) {
+	// Nothing to do for this context;
+}
+
 public void traverse(ASTVisitor visitor, CompilationUnitScope unitScope) {
-	if (this.ignoreFurtherInvestigation)
+	traverse(visitor, unitScope, true);
+}
+public void traverse(ASTVisitor visitor, CompilationUnitScope unitScope, boolean skipOnError) {
+	if (skipOnError && this.ignoreFurtherInvestigation)
 		return;
 	try {
 		if (visitor.visit(this, this.scope)) {
