@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
+ * Copyright (c) 2000, 2014 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -15,9 +15,16 @@
  *     						Bug 358903 - Filter practically unimportant resource leak warnings
  *							Bug 395977 - [compiler][resource] Resource leak warning behavior possibly incorrect for anonymous inner class
  *							Bug 395002 - Self bound generic class doesn't resolve bounds properly for wildcards for certain parametrisation.
+ *							Bug 416176 - [1.8][compiler][null] null type annotations cause grief on type variables
+ *							Bug 427199 - [1.8][resource] avoid resource leak warnings on Streams that have no resource
+ *							Bug 429958 - [1.8][null] evaluate new DefaultLocation attribute of @NonNullByDefault
+ *							Bug 434570 - Generic type mismatch for parametrized class annotation attribute with inner class
+ *        Andy Clement (GoPivotal, Inc) aclement@gopivotal.com - Contributions for
+ *                          Bug 415821 - [1.8][compiler] CLASS_EXTENDS target type annotation missing for anonymous classes
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.lookup;
 
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
@@ -37,11 +44,12 @@ import org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
 import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
 import org.eclipse.jdt.internal.compiler.util.HashtableOfObject;
 
+@SuppressWarnings({"rawtypes"})
 public class ClassScope extends Scope {
 
 	public TypeDeclaration referenceContext;
 	public TypeReference superTypeReference;
-	java.util.ArrayList deferredBoundChecks;
+	java.util.ArrayList<Object> deferredBoundChecks; // contains TypeReference or Runnable. TODO consider making this a List<Runnable>
 
 	public ClassScope(Scope parent, TypeDeclaration context) {
 		super(Scope.CLASS_SCOPE, parent);
@@ -67,33 +75,35 @@ public class ClassScope extends Scope {
 		}
 		anonymousType.typeBits |= inheritedBits;
 		if (supertype.isInterface()) {
-			anonymousType.superclass = getJavaLangObject();
-			anonymousType.superInterfaces = new ReferenceBinding[] { supertype };
+			anonymousType.setSuperClass(getJavaLangObject());
+			anonymousType.setSuperInterfaces(new ReferenceBinding[] { supertype });
 			TypeReference typeReference = this.referenceContext.allocation.type;
 			if (typeReference != null) {
+				this.referenceContext.superInterfaces = new TypeReference[] { typeReference };
 				if ((supertype.tagBits & TagBits.HasDirectWildcard) != 0) {
 					problemReporter().superTypeCannotUseWildcard(anonymousType, typeReference, supertype);
 					anonymousType.tagBits |= TagBits.HierarchyHasProblems;
-					anonymousType.superInterfaces = Binding.NO_SUPERINTERFACES;
+					anonymousType.setSuperInterfaces(Binding.NO_SUPERINTERFACES);
 				}
 			}
 		} else {
-			anonymousType.superclass = supertype;
-			anonymousType.superInterfaces = Binding.NO_SUPERINTERFACES;
+			anonymousType.setSuperClass(supertype);
+			anonymousType.setSuperInterfaces(Binding.NO_SUPERINTERFACES);
 			TypeReference typeReference = this.referenceContext.allocation.type;
 			if (typeReference != null) { // no check for enum constant body
+				this.referenceContext.superclass = typeReference;
 				if (supertype.erasure().id == TypeIds.T_JavaLangEnum) {
 					problemReporter().cannotExtendEnum(anonymousType, typeReference, supertype);
 					anonymousType.tagBits |= TagBits.HierarchyHasProblems;
-					anonymousType.superclass = getJavaLangObject();
+					anonymousType.setSuperClass(getJavaLangObject());
 				} else if (supertype.isFinal()) {
 					problemReporter().anonymousClassCannotExtendFinalClass(typeReference, supertype);
 					anonymousType.tagBits |= TagBits.HierarchyHasProblems;
-					anonymousType.superclass = getJavaLangObject();
+					anonymousType.setSuperClass(getJavaLangObject());
 				} else if ((supertype.tagBits & TagBits.HasDirectWildcard) != 0) {
 					problemReporter().superTypeCannotUseWildcard(anonymousType, typeReference, supertype);
 					anonymousType.tagBits |= TagBits.HierarchyHasProblems;
-					anonymousType.superclass = getJavaLangObject();
+					anonymousType.setSuperClass(getJavaLangObject());
 				}
 			}
 		}
@@ -231,7 +241,7 @@ public class ClassScope extends Scope {
 			if (count != size)
 				System.arraycopy(memberTypeBindings, 0, memberTypeBindings = new ReferenceBinding[count], 0, count);
 		}
-		localType.memberTypes = memberTypeBindings;
+		localType.setMemberTypes(memberTypeBindings);
 		return localType;
 	}
 
@@ -292,7 +302,7 @@ public class ClassScope extends Scope {
 			if (count != length)
 				System.arraycopy(memberTypeBindings, 0, memberTypeBindings = new ReferenceBinding[count], 0, count);
 		}
-		sourceType.memberTypes = memberTypeBindings;
+		sourceType.setMemberTypes(memberTypeBindings);
 	}
 
 	void buildMethods() {
@@ -398,9 +408,13 @@ public class ClassScope extends Scope {
 
 		SourceTypeBinding sourceType = this.referenceContext.binding;
 		environment().setAccessRestriction(sourceType, accessRestriction);
+		
+		TypeParameter[] typeParameters = this.referenceContext.typeParameters;
+		sourceType.typeVariables = typeParameters == null || typeParameters.length == 0 ? Binding.NO_TYPE_VARIABLES : null;
 		sourceType.fPackage.addType(sourceType);
 		checkAndSetModifiers();
 		buildTypeVariables();
+		
 		buildMemberTypes(accessRestriction);
 		return sourceType;
 	}
@@ -411,19 +425,23 @@ public class ClassScope extends Scope {
 		TypeParameter[] typeParameters = this.referenceContext.typeParameters;
 		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=324850, If they exist at all, process type parameters irrespective of source level.
 		if (typeParameters == null || typeParameters.length == 0) {
-		    sourceType.typeVariables = Binding.NO_TYPE_VARIABLES;
+		    sourceType.setTypeVariables(Binding.NO_TYPE_VARIABLES);
 		    return;
 		}
-		sourceType.typeVariables = Binding.NO_TYPE_VARIABLES; // safety
+		sourceType.setTypeVariables(Binding.NO_TYPE_VARIABLES); // safety
 
 		if (sourceType.id == TypeIds.T_JavaLangObject) { // handle the case of redefining java.lang.Object up front
 			problemReporter().objectCannotBeGeneric(this.referenceContext);
 			return;
 		}
-		sourceType.typeVariables = createTypeVariables(typeParameters, sourceType);
+		sourceType.setTypeVariables(createTypeVariables(typeParameters, sourceType));
 		sourceType.modifiers |= ExtraCompilerModifiers.AccGenericSignature;
 	}
 
+	void resolveTypeParameter(TypeParameter typeParameter) {
+		typeParameter.resolve(this);
+	}
+
 	private void checkAndSetModifiers() {
 		SourceTypeBinding sourceType = this.referenceContext.binding;
 		int modifiers = sourceType.modifiers;
@@ -441,6 +459,8 @@ public class ClassScope extends Scope {
 					problemReporter().nonStaticContextForEnumMemberType(sourceType);
 				else
 					modifiers |= ClassFileConstants.AccStatic;
+			} else if (sourceType.isInterface()) {
+				modifiers |= ClassFileConstants.AccStatic; // 8.5.1
 			}
 		} else if (sourceType.isLocalType()) {
 			if (sourceType.isEnum()) {
@@ -459,6 +479,8 @@ public class ClassScope extends Scope {
 				switch (scope.kind) {
 					case METHOD_SCOPE :
 						MethodScope methodScope = (MethodScope) scope;
+						if (methodScope.isLambdaScope()) 
+							methodScope = methodScope.namedMethodScope();
 						if (methodScope.isInsideInitializer()) {
 							SourceTypeBinding type = ((TypeDeclaration) methodScope.referenceContext).binding;
 
@@ -766,7 +788,7 @@ public class ClassScope extends Scope {
 
 			TypeReference[] boundRefs = typeParameter.bounds;
 			if (boundRefs != null) {
-				boolean checkSuperclass = typeVariable.firstBound == typeVariable.superclass;
+				boolean checkSuperclass = TypeBinding.equalsEquals(typeVariable.firstBound, typeVariable.superclass);
 				for (int j = 0, boundLength = boundRefs.length; j < boundLength; j++) {
 					TypeReference typeRef = boundRefs[j];
 					TypeBinding superType = typeRef.resolvedType;
@@ -814,7 +836,7 @@ public class ClassScope extends Scope {
 					nextInterface : for (int a = 0; a < itsLength; a++) {
 						ReferenceBinding next = itsInterfaces[a];
 						for (int b = 0; b < nextPosition; b++)
-							if (next == interfacesToVisit[b]) continue nextInterface;
+							if (TypeBinding.equalsEquals(next, interfacesToVisit[b])) continue nextInterface;
 						interfacesToVisit[nextPosition++] = next;
 					}
 				}
@@ -839,7 +861,7 @@ public class ClassScope extends Scope {
 						nextInterface : for (int a = 0; a < itsLength; a++) {
 							ReferenceBinding next = itsInterfaces[a];
 							for (int b = 0; b < nextPosition; b++)
-								if (next == interfacesToVisit[b]) continue nextInterface;
+								if (TypeBinding.equalsEquals(next, interfacesToVisit[b])) continue nextInterface;
 							interfacesToVisit[nextPosition++] = next;
 						}
 					}
@@ -861,8 +883,13 @@ public class ClassScope extends Scope {
 
 	// Perform deferred bound checks for parameterized type references (only done after hierarchy is connected)
 	public void  checkParameterizedTypeBounds() {
-		for (int i = 0, l = this.deferredBoundChecks == null ? 0 : this.deferredBoundChecks.size(); i < l; i++)
-			((TypeReference) this.deferredBoundChecks.get(i)).checkBounds(this);
+		for (int i = 0, l = this.deferredBoundChecks == null ? 0 : this.deferredBoundChecks.size(); i < l; i++) {
+			Object toCheck = this.deferredBoundChecks.get(i);
+			if (toCheck instanceof TypeReference)
+				((TypeReference) toCheck).checkBounds(this);
+			else if (toCheck instanceof Runnable)
+				((Runnable) toCheck).run();
+		}
 		this.deferredBoundChecks = null;
 
 		ReferenceBinding[] memberTypes = this.referenceContext.binding.memberTypes;
@@ -893,8 +920,8 @@ public class ClassScope extends Scope {
 	private boolean connectSuperclass() {
 		SourceTypeBinding sourceType = this.referenceContext.binding;
 		if (sourceType.id == TypeIds.T_JavaLangObject) { // handle the case of redefining java.lang.Object up front
-			sourceType.superclass = null;
-			sourceType.superInterfaces = Binding.NO_SUPERINTERFACES;
+			sourceType.setSuperClass(null);
+			sourceType.setSuperInterfaces(Binding.NO_SUPERINTERFACES);
 			if (!sourceType.isClass())
 				problemReporter().objectMustBeClass(sourceType);
 			if (this.referenceContext.superclass != null || (this.referenceContext.superInterfaces != null && this.referenceContext.superInterfaces.length > 0))
@@ -904,7 +931,7 @@ public class ClassScope extends Scope {
 		if (this.referenceContext.superclass == null) {
 			if (sourceType.isEnum() && compilerOptions().sourceLevel >= ClassFileConstants.JDK1_5) // do not connect if source < 1.5 as enum already got flagged as syntax error
 				return connectEnumSuperclass();
-			sourceType.superclass = getJavaLangObject();
+			sourceType.setSuperClass(getJavaLangObject());
 			return !detectHierarchyCycle(sourceType, sourceType.superclass, null);
 		}
 		TypeReference superclassRef = this.referenceContext.superclass;
@@ -920,21 +947,21 @@ public class ClassScope extends Scope {
 				problemReporter().cannotExtendEnum(sourceType, superclassRef, superclass);
 			} else if ((superclass.tagBits & TagBits.HierarchyHasProblems) != 0
 					|| !superclassRef.resolvedType.isValidBinding()) {
-				sourceType.superclass = superclass;
+				sourceType.setSuperClass(superclass);
 				sourceType.tagBits |= TagBits.HierarchyHasProblems; // propagate if missing supertype
 				return superclassRef.resolvedType.isValidBinding(); // reported some error against the source type ?
 			} else {
 				// only want to reach here when no errors are reported
-				sourceType.superclass = superclass;
+				sourceType.setSuperClass(superclass);
 				sourceType.typeBits |= (superclass.typeBits & TypeIds.InheritableBits);
 				// further analysis against white lists for the unlikely case we are compiling java.io.*:
 				if ((sourceType.typeBits & (TypeIds.BitAutoCloseable|TypeIds.BitCloseable)) != 0)
-					sourceType.typeBits |= sourceType.applyCloseableWhitelists();
+					sourceType.typeBits |= sourceType.applyCloseableClassWhitelists();
 				return true;
 			}
 		}
 		sourceType.tagBits |= TagBits.HierarchyHasProblems;
-		sourceType.superclass = getJavaLangObject();
+		sourceType.setSuperClass(getJavaLangObject());
 		if ((sourceType.superclass.tagBits & TagBits.BeginHierarchyCheck) == 0)
 			detectHierarchyCycle(sourceType, sourceType.superclass, null);
 		return false; // reported some error against the source type
@@ -948,7 +975,7 @@ public class ClassScope extends Scope {
 		ReferenceBinding rootEnumType = getJavaLangEnum();
 		if ((rootEnumType.tagBits & TagBits.HasMissingType) != 0) {
 			sourceType.tagBits |= TagBits.HierarchyHasProblems; // mark missing supertpye
-			sourceType.superclass = rootEnumType;
+			sourceType.setSuperClass(rootEnumType);
 			return false;
 		}
 		boolean foundCycle = detectHierarchyCycle(sourceType, rootEnumType, null);
@@ -969,7 +996,7 @@ public class ClassScope extends Scope {
 			} ,
 			null);
 		sourceType.tagBits |= (superType.tagBits & TagBits.HierarchyHasProblems); // propagate if missing supertpye
-		sourceType.superclass = superType;
+		sourceType.setSuperClass(superType);
 		// bound check (in case of bogus definition of Enum type)
 		if (refTypeVariables[0].boundCheck(superType, sourceType, this) != TypeConstants.OK) {
 			problemReporter().typeMismatchError(rootEnumType, refTypeVariables[0], sourceType, null);
@@ -989,12 +1016,12 @@ public class ClassScope extends Scope {
 	*/
 	private boolean connectSuperInterfaces() {
 		SourceTypeBinding sourceType = this.referenceContext.binding;
-		sourceType.superInterfaces = Binding.NO_SUPERINTERFACES;
+		sourceType.setSuperInterfaces(Binding.NO_SUPERINTERFACES);
 		if (this.referenceContext.superInterfaces == null) {
 			if (sourceType.isAnnotationType() && compilerOptions().sourceLevel >= ClassFileConstants.JDK1_5) { // do not connect if source < 1.5 as annotation already got flagged as syntax error) {
 				ReferenceBinding annotationType = getJavaLangAnnotationAnnotation();
 				boolean foundCycle = detectHierarchyCycle(sourceType, annotationType, null);
-				sourceType.superInterfaces = new ReferenceBinding[] { annotationType };
+				sourceType.setSuperInterfaces(new ReferenceBinding[] { annotationType });
 				return !foundCycle;
 			}
 			return true;
@@ -1018,7 +1045,7 @@ public class ClassScope extends Scope {
 			// check for simple interface collisions
 			// Check for a duplicate interface once the name is resolved, otherwise we may be confused (i.e. a.b.I and c.d.I)
 			for (int j = 0; j < i; j++) {
-				if (interfaceBindings[j] == superInterface) {
+				if (TypeBinding.equalsEquals(interfaceBindings[j], superInterface)) {
 					problemReporter().duplicateSuperinterface(sourceType, superInterfaceRef, superInterface);
 					sourceType.tagBits |= TagBits.HierarchyHasProblems;
 					noProblems = false;
@@ -1046,13 +1073,16 @@ public class ClassScope extends Scope {
 			}
 			// only want to reach here when no errors are reported
 			sourceType.typeBits |= (superInterface.typeBits & TypeIds.InheritableBits);
+			// further analysis against white lists for the unlikely case we are compiling java.util.stream.Stream:
+			if ((sourceType.typeBits & (TypeIds.BitAutoCloseable|TypeIds.BitCloseable)) != 0)
+				sourceType.typeBits |= sourceType.applyCloseableInterfaceWhitelists();
 			interfaceBindings[count++] = superInterface;
 		}
 		// hold onto all correctly resolved superinterfaces
 		if (count > 0) {
 			if (count != length)
 				System.arraycopy(interfaceBindings, 0, interfaceBindings = new ReferenceBinding[count], 0, count);
-			sourceType.superInterfaces = interfaceBindings;
+			sourceType.setSuperInterfaces(interfaceBindings);
 		}
 		return noProblems;
 	}
@@ -1084,6 +1114,18 @@ public class ClassScope extends Scope {
 		}
 	}
 
+	@Override
+	public boolean deferCheck(Runnable check) {
+		if (compilationUnitScope().connectingHierarchy) {
+			if (this.deferredBoundChecks == null)
+				this.deferredBoundChecks = new ArrayList<Object>();
+			this.deferredBoundChecks.add(check);
+			return true;
+		} else {
+			return false;
+		}
+	}
+
 	private void connectTypeHierarchyWithoutMembers() {
 		// must ensure the imports are resolved
 		if (this.parent instanceof CompilationUnitScope) {
@@ -1139,7 +1181,7 @@ public class ClassScope extends Scope {
 			superType = ((RawTypeBinding) superType).genericType();
 		// by this point the superType must be a binary or source type
 
-		if (sourceType == superType) {
+		if (TypeBinding.equalsEquals(sourceType, superType)) {
 			problemReporter().hierarchyCircularity(sourceType, superType, reference);
 			sourceType.tagBits |= TagBits.HierarchyHasProblems;
 			return true;
@@ -1148,7 +1190,7 @@ public class ClassScope extends Scope {
 		if (superType.isMemberType()) {
 			ReferenceBinding current = superType.enclosingType();
 			do {
-				if (current.isHierarchyBeingActivelyConnected() && current == sourceType) {
+				if (current.isHierarchyBeingActivelyConnected() && TypeBinding.equalsEquals(current, sourceType)) {
 					problemReporter().hierarchyCircularity(sourceType, current, reference);
 					sourceType.tagBits |= TagBits.HierarchyHasProblems;
 					current.tagBits |= TagBits.HierarchyHasProblems;
@@ -1164,7 +1206,7 @@ public class ClassScope extends Scope {
 			boolean hasCycle = false;
 			ReferenceBinding parentType = superType.superclass();
 			if (parentType != null) {
-				if (sourceType == parentType) {
+				if (TypeBinding.equalsEquals(sourceType, parentType)) {
 					problemReporter().hierarchyCircularity(sourceType, superType, reference);
 					sourceType.tagBits |= TagBits.HierarchyHasProblems;
 					superType.tagBits |= TagBits.HierarchyHasProblems;
@@ -1183,7 +1225,7 @@ public class ClassScope extends Scope {
 			if (itsInterfaces != null && itsInterfaces != Binding.NO_SUPERINTERFACES) {
 				for (int i = 0, length = itsInterfaces.length; i < length; i++) {
 					ReferenceBinding anInterface = itsInterfaces[i];
-					if (sourceType == anInterface) {
+					if (TypeBinding.equalsEquals(sourceType, anInterface)) {
 						problemReporter().hierarchyCircularity(sourceType, superType, reference);
 						sourceType.tagBits |= TagBits.HierarchyHasProblems;
 						superType.tagBits |= TagBits.HierarchyHasProblems;
@@ -1246,7 +1288,7 @@ public class ClassScope extends Scope {
 			return superType;
 		} catch (AbortCompilation e) {
 			SourceTypeBinding sourceType = this.referenceContext.binding;
-			if (sourceType.superInterfaces == null)  sourceType.superInterfaces = Binding.NO_SUPERINTERFACES; // be more resilient for hierarchies (144976)
+			if (sourceType.superInterfaces == null)  sourceType.setSuperInterfaces(Binding.NO_SUPERINTERFACES); // be more resilient for hierarchies (144976)
 			e.updateContext(typeReference, referenceCompilationUnit().compilationResult);
 			throw e;
 		} finally {
@@ -1278,6 +1320,17 @@ public class ClassScope extends Scope {
 		return this.referenceContext;
 	}
 
+	@Override
+	public boolean hasDefaultNullnessFor(int location) {
+		SourceTypeBinding binding = this.referenceContext.binding;
+		if (binding != null) {
+			int nullDefault = binding.getNullDefault();
+			if (nullDefault != 0)
+				return (nullDefault & location) != 0;
+		}
+		return this.parent.hasDefaultNullnessFor(location);
+	}
+
 	public String toString() {
 		if (this.referenceContext != null)
 			return "--- Class Scope ---\n\n"  //$NON-NLS-1$
