@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2008 IBM Corporation and others.
+ * Copyright (c) 2000, 2014 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,9 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contributions for
+ *								Bug 186342 - [compiler][null] Using annotations for null checking
+ *								Bug 429958 - [1.8][null] evaluate new DefaultLocation attribute of @NonNullByDefault
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.lookup;
 
@@ -27,6 +30,8 @@ public class MostSpecificExceptionMethodBinding  extends MethodBinding {
 				mostSpecificExceptions, 
 				originalMethod.declaringClass);
 		this.originalMethod = originalMethod;
+		this.parameterNonNullness = originalMethod.parameterNonNullness;
+		this.defaultNullness = originalMethod.defaultNullness;
 	}
 	
 	public MethodBinding original() {
