@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * Copyright (c) 2000, 2014 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,14 +7,18 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contribution for
+ *							Bug 400874 - [1.8][compiler] Inference infrastructure should evolve to meet JLS8 18.x (Part G of JSR335 spec)
+ *							Bug 433478 - [compiler][null] NPE in ReferenceBinding.isCompatibleWith
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.lookup;
 
 public class ProblemMethodBinding extends MethodBinding {
-    
+
 	private int problemReason;
 	public MethodBinding closestMatch; // TODO (philippe) should rename into #alternateMatch
-	
+	public InferenceContext18 inferenceContext; // inference context may help to coordinate error reporting
+
 public ProblemMethodBinding(char[] selector, TypeBinding[] args, int problemReason) {
 	this.selector = selector;
 	this.parameters = (args == null || args.length == 0) ? Binding.NO_PARAMETERS : args;
@@ -31,7 +35,10 @@ public ProblemMethodBinding(char[] selector, TypeBinding[] args, ReferenceBindin
 public ProblemMethodBinding(MethodBinding closestMatch, char[] selector, TypeBinding[] args, int problemReason) {
 	this(selector, args, problemReason);
 	this.closestMatch = closestMatch;
-	if (closestMatch != null && problemReason != ProblemReasons.Ambiguous) this.declaringClass = closestMatch.declaringClass;
+	if (closestMatch != null && problemReason != ProblemReasons.Ambiguous) {
+		this.declaringClass = closestMatch.declaringClass;
+		this.returnType = closestMatch.returnType;
+	}
 }
 /* API
 * Answer the problem id associated with the receiver.
