@@ -14,6 +14,8 @@
  *								bug 358903 - Filter practically unimportant resource leak warnings
  *								bug 400421 - [compiler] Null analysis for fields does not take @com.google.inject.Inject into account
  *								bug 382069 - [null] Make the null analysis consider JUnit's assertNotNull similarly to assertions
+ *      Jesper S Moller <jesper@selskabet.org> -  Contributions for
+ *								Bug 412153 - [1.8][compiler] Check validity of annotations which may be repeatable
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.lookup;
 
@@ -119,11 +121,17 @@ public interface TypeIds {
 	final int T_ComGoogleCommonBasePreconditions = 73;
 	final int T_JavaUtilObjects = 74;
 
+	// java 8
+	final int T_JavaLangFunctionalInterface = 77;
+
 	// new in 3.9 to identify known @Inject annotations
 	final int T_JavaxInjectInject = 80;
 	final int T_ComGoogleInjectInject = 81;
-
-
+	// Java 8 - JEP 120
+	final int T_JavaLangAnnotationRepeatable = 90;
+	// If you add new type id, make sure to bump up T_LastWellKnownTypeId if there is a cross over.
+	final int T_LastWellKnownTypeId = 128;
+	
 	final int NoId = Integer.MAX_VALUE;
 
 	public static final int IMPLICIT_CONVERSION_MASK = 0xFF;
@@ -231,8 +239,9 @@ public interface TypeIds {
 	 */
 	final int BitResourceFreeCloseable = 8;
 	
+	final int BitUninternedType = 16;
 	/**
 	 * Set of type bits that should be inherited by any sub types.
 	 */
-	final int InheritableBits = BitAutoCloseable | BitCloseable;
+	final int InheritableBits = BitAutoCloseable | BitCloseable | BitUninternedType;
 }
